import datetime
import logging
import pytz
import warnings
import pandas as pd
from psycopg2 import connect
from ..y4a_credentials import get_credentials

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

warnings.filterwarnings("ignore", category=UserWarning)


class MonitorUserExternal:
    def __init__(
        self,
        conn_username: str,
        conn_password: str,
        conn_host: str,
        conn_db: str,
        conn_number: int,
    ) -> None:
        self.conn_username = conn_username
        self.conn_password = conn_password
        self.conn_host = conn_host
        self.conn_db = conn_db
        self.conn_number = conn_number

        self.log_conn_df = pd.DataFrame(
            {
                'time_conn': str(
                    datetime.datetime.now(
                        pytz.timezone('Asia/Ho_Chi_Minh'),
                    ).replace(microsecond=0).replace(tzinfo=None)
                ),
                'host': self.conn_host,
                'username': self.conn_username,
                'password': self.conn_password,
                'database': self.conn_db,
                'num_conn': self.conn_number,
            },
            index=[0],
        )

    def execute(
        self,
        sql: str,
        vars: tuple = None,
        fetch_output: bool = False,
    ) -> list:
        output = None

        credentials = get_credentials(
            platform='pg',
            account_name='clong',
        )

        conn = connect(
            user=credentials['user_name'],
            password=credentials['password'],
            host='172.30.105.100',
            port=5432,
            database='y4a_datawarehouse',
        )

        try:
            cur = conn.cursor()

            cur.execute(sql, vars)

            if fetch_output:
                output = cur.fetchall()

            cur.close()
            conn.commit()
            conn.close()
        except Exception as e:
            logging.error(e)
            conn.close()

        return output

    def generate_log_conn(self) -> None:
        sql = "INSERT INTO y4a_sop.monitor_user_sql_connection "
        sql += "(time_conn, host, username, database, num_conn) "
        sql += f"VALUES ('{self.log_conn_df['time_conn'].values[0]}', "
        sql += f"'{self.log_conn_df['host'].values[0]}', "
        sql += f"'{self.log_conn_df['username'].values[0]}', "
        sql += f"'{self.log_conn_df['database'].values[0]}', "
        sql += f"{self.log_conn_df['num_conn'].values[0]}) "

        self.execute(
            sql=sql,
            fetch_output=False,
        )

    def generate_log_query(
        self,
        sql_query: str,
        duration_query: float,
        is_successed: int,
    ) -> None:
        sql = "INSERT INTO y4a_sop.monitor_user_sql_query "
        sql += "(time_query, host, username, database, "
        sql += "sql_query, duration, is_successed) "
        sql += f"VALUES ('{self.log_conn_df['time_conn'].values[0]}', "
        sql += f"'{self.log_conn_df['host'].values[0]}', "
        sql += f"'{self.log_conn_df['username'].values[0]}', "
        sql += f"'{self.log_conn_df['database'].values[0]}', "
        sql += "%s, "
        sql += f"{duration_query}, "
        sql += f"{is_successed}) "

        self.execute(
            sql=sql,
            vars=(
                sql_query,
            ),
            fetch_output=False,
        )



import requests
import logging
import warnings
from io import BytesIO
from minio import Minio
import pandas as pd

warnings.filterwarnings("ignore", category=UserWarning)

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)


def get_credentials(
    platform: str,
    account_name: str,
) -> None:
    try:
        response = requests.get(
            f'http://172.30.15.171:5431/v1/users/credentials'
            f'/{platform}/{account_name}'
        ).json()
    except Exception as e:
        logging.error(f'Failed to get account credentials: {e}')
        logging.info('Try to get account credentials from minIO')
        client = Minio(
            endpoint='minio-raw-api.yes4all.internal',
            access_key='sop_Y7Suhzcuanc1Jkdr',
            secret_key='sjVOaPYb7yhFekxlkM0mdOWehiZgkIHIBfq9Xere',
            secure=False,
        )
        parquet_data = BytesIO(
            client.get_object(
                bucket_name='sop-bucket',
                object_name='prod/creds.parquet',
            ).data
        )
        df = pd.read_parquet(parquet_data)

        response = df.loc[
            (df['platform'] == platform)
            & (df['account_name'] == account_name)
        ]['values'].values[0]

    return response



import logging
import warnings
import functools
import time

warnings.filterwarnings("ignore", category=UserWarning)

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)


def retry_on_error(
    max_retries: int = 3,
    delay: int = 1,
):
    def decorator_retry(func):
        @functools.wraps(func)
        def wrapper_retry(*args, **kwargs):
            attempts = 0
            while attempts < max_retries:
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    logging.error(e)
                    attempts += 1
                    if attempts < max_retries:
                        logging.info(f'Retrying in {delay} seconds...')
                        time.sleep(delay)
                    else:
                        logging.info('Maximum retries reached.')
                        raise
        return wrapper_retry
    return decorator_retry
